@@ -13,8 +13,7 @@ from google.oauth2 import service_account
 from google.oauth2.credentials import Credentials as OAuthCredentials
 
 from ray.autoscaler._private.util import check_legacy_fields
-from ray.autoscaler._private.gcp.node import (GCPNodeType, MAX_POLLS,
-                                              POLL_INTERVAL)
+from ray.autoscaler._private.gcp.node import GCPNodeType, MAX_POLLS, POLL_INTERVAL
 
 logger = logging.getLogger(__name__)
 
@@ -23,16 +22,19 @@ TPU_VERSION = "v2alpha"  # change once v2 is stable
 
 RAY = "ray-autoscaler"
 DEFAULT_SERVICE_ACCOUNT_ID = RAY + "-sa-" + VERSION
-SERVICE_ACCOUNT_EMAIL_TEMPLATE = (
-    "{account_id}@{project_id}.iam.gserviceaccount.com")
+SERVICE_ACCOUNT_EMAIL_TEMPLATE = "{account_id}@{project_id}.iam.gserviceaccount.com"
 DEFAULT_SERVICE_ACCOUNT_CONFIG = {
     "displayName": "Ray Autoscaler Service Account ({})".format(VERSION),
 }
 
 # Those roles will be always added.
+# NOTE: `serviceAccountUser` allows the head node to create workers with
+# a serviceAccount. `roleViewer` allows the head node to run bootstrap_gcp.
 DEFAULT_SERVICE_ACCOUNT_ROLES = [
-    "roles/storage.objectAdmin", "roles/compute.admin",
-    "roles/iam.serviceAccountUser"
+    "roles/storage.objectAdmin",
+    "roles/compute.admin",
+    "roles/iam.serviceAccountUser",
+    "roles/iam.roleViewer",
 ]
 # Those roles will only be added if there are TPU nodes defined in config.
 TPU_SERVICE_ACCOUNT_ROLES = ["roles/tpu.admin"]
@@ -62,14 +64,16 @@ def get_node_type(node: dict) -> GCPNodeType:
             "required. "
             "For a TPU instance, 'acceleratorType' and no 'machineType' "
             "is required. "
-            f"Got {list(node)}")
+            f"Got {list(node)}"
+        )
 
     if "machineType" not in node and "acceleratorType" in node:
         # remove after TPU pod support is added!
         if node["acceleratorType"] not in ("v2-8", "v3-8"):
             raise ValueError(
                 "For now, only v2-8' and 'v3-8' accelerator types are "
-                "supported. Support for TPU pods will be added in the future.")
+                "supported. Support for TPU pods will be added in the future."
+            )
 
         return GCPNodeType.TPU
     return GCPNodeType.COMPUTE
@@ -77,8 +81,10 @@ def get_node_type(node: dict) -> GCPNodeType:
 
 def wait_for_crm_operation(operation, crm):
     """Poll for cloud resource manager operation until finished."""
-    logger.info("wait_for_crm_operation: "
-                "Waiting for operation {} to finish...".format(operation))
+    logger.info(
+        "wait_for_crm_operation: "
+        "Waiting for operation {} to finish...".format(operation)
+    )
 
     for _ in range(MAX_POLLS):
         result = crm.operations().get(name=operation["name"]).execute()
@@ -96,21 +102,25 @@ def wait_for_crm_operation(operation, crm):
 
 def wait_for_compute_global_operation(project_name, operation, compute):
     """Poll for global compute operation until finished."""
-    logger.info("wait_for_compute_global_operation: "
-                "Waiting for operation {} to finish...".format(
-                    operation["name"]))
+    logger.info(
+        "wait_for_compute_global_operation: "
+        "Waiting for operation {} to finish...".format(operation["name"])
+    )
 
     for _ in range(MAX_POLLS):
-        result = compute.globalOperations().get(
-            project=project_name,
-            operation=operation["name"],
-        ).execute()
+        result = (
+            compute.globalOperations()
+            .get(
+                project=project_name,
+                operation=operation["name"],
+            )
+            .execute()
+        )
         if "error" in result:
             raise Exception(result["error"])
 
         if result["status"] == "DONE":
-            logger.info("wait_for_compute_global_operation: "
-                        "Operation done.")
+            logger.info("wait_for_compute_global_operation: Operation done.")
             break
 
         time.sleep(POLL_INTERVAL)
@@ -120,8 +130,7 @@ def wait_for_compute_global_operation(project_name, operation, compute):
 
 def key_pair_name(i, region, project_id, ssh_user):
     """Returns the ith default gcp_key_pair_name."""
-    key_name = "{}_gcp_{}_{}_{}_{}".format(RAY, region, project_id, ssh_user,
-                                           i)
+    key_name = "{}_gcp_{}_{}_{}_{}".format(RAY, region, project_id, ssh_user, i)
     return key_name
 
 
@@ -136,16 +145,22 @@ def generate_rsa_key_pair():
     """Create public and private ssh-keys."""
 
     key = rsa.generate_private_key(
-        backend=default_backend(), public_exponent=65537, key_size=2048)
+        backend=default_backend(), public_exponent=65537, key_size=2048
+    )
 
-    public_key = key.public_key().public_bytes(
-        serialization.Encoding.OpenSSH,
-        serialization.PublicFormat.OpenSSH).decode("utf-8")
+    public_key = (
+        key.public_key()
+        .public_bytes(
+            serialization.Encoding.OpenSSH, serialization.PublicFormat.OpenSSH
+        )
+        .decode("utf-8")
+    )
 
     pem = key.private_bytes(
         encoding=serialization.Encoding.PEM,
         format=serialization.PrivateFormat.TraditionalOpenSSL,
-        encryption_algorithm=serialization.NoEncryption()).decode("utf-8")
+        encryption_algorithm=serialization.NoEncryption(),
+    ).decode("utf-8")
 
     return public_key, pem
 
@@ -165,26 +180,25 @@ def _is_head_node_a_tpu(config: dict) -> bool:
         node_id: node_type["node_config"]
         for node_id, node_type in config["available_node_types"].items()
     }
-    return get_node_type(
-        node_configs[config["head_node_type"]]) == GCPNodeType.TPU
+    return get_node_type(node_configs[config["head_node_type"]]) == GCPNodeType.TPU
 
 
 def _create_crm(gcp_credentials=None):
     return discovery.build(
-        "cloudresourcemanager",
-        "v1",
-        credentials=gcp_credentials,
-        cache_discovery=False)
+        "cloudresourcemanager", "v1", credentials=gcp_credentials, cache_discovery=False
+    )
 
 
 def _create_iam(gcp_credentials=None):
     return discovery.build(
-        "iam", "v1", credentials=gcp_credentials, cache_discovery=False)
+        "iam", "v1", credentials=gcp_credentials, cache_discovery=False
+    )
 
 
 def _create_compute(gcp_credentials=None):
     return discovery.build(
-        "compute", "v1", credentials=gcp_credentials, cache_discovery=False)
+        "compute", "v1", credentials=gcp_credentials, cache_discovery=False
+    )
 
 
 def _create_tpu(gcp_credentials=None):
@@ -193,7 +207,8 @@ def _create_tpu(gcp_credentials=None):
         TPU_VERSION,
         credentials=gcp_credentials,
         cache_discovery=False,
-        discoveryServiceUrl="https://tpu.googleapis.com/$discovery/rest")
+        discoveryServiceUrl="https://tpu.googleapis.com/$discovery/rest",
+    )
 
 
 def construct_clients_from_provider_config(provider_config):
@@ -206,22 +221,26 @@ def construct_clients_from_provider_config(provider_config):
     """
     gcp_credentials = provider_config.get("gcp_credentials")
     if gcp_credentials is None:
-        logger.debug("gcp_credentials not found in cluster yaml file. "
-                     "Falling back to GOOGLE_APPLICATION_CREDENTIALS "
-                     "environment variable.")
-        tpu_resource = _create_tpu() if provider_config.get(
-            HAS_TPU_PROVIDER_FIELD, False) else None
+        logger.debug(
+            "gcp_credentials not found in cluster yaml file. "
+            "Falling back to GOOGLE_APPLICATION_CREDENTIALS "
+            "environment variable."
+        )
+        tpu_resource = (
+            _create_tpu()
+            if provider_config.get(HAS_TPU_PROVIDER_FIELD, False)
+            else None
+        )
         # If gcp_credentials is None, then discovery.build will search for
         # credentials in the local environment.
-        return _create_crm(), \
-            _create_iam(), \
-            _create_compute(), \
-            tpu_resource
+        return _create_crm(), _create_iam(), _create_compute(), tpu_resource
 
-    assert ("type" in gcp_credentials), \
-        "gcp_credentials cluster yaml field missing 'type' field."
-    assert ("credentials" in gcp_credentials), \
-        "gcp_credentials cluster yaml field missing 'credentials' field."
+    assert (
+        "type" in gcp_credentials
+    ), "gcp_credentials cluster yaml field missing 'type' field."
+    assert (
+        "credentials" in gcp_credentials
+    ), "gcp_credentials cluster yaml field missing 'credentials' field."
 
     cred_type = gcp_credentials["type"]
     credentials_field = gcp_credentials["credentials"]
@@ -234,20 +253,27 @@ def construct_clients_from_provider_config(provider_config):
         except json.decoder.JSONDecodeError:
             raise RuntimeError(
                 "gcp_credentials found in cluster yaml file but "
-                "formatted improperly.")
+                "formatted improperly."
+            )
         credentials = service_account.Credentials.from_service_account_info(
-            service_account_info)
+            service_account_info
+        )
     elif cred_type == "credentials_token":
         # Otherwise the credentials type must be credentials_token.
         credentials = OAuthCredentials(credentials_field)
 
-    tpu_resource = _create_tpu(credentials) if provider_config.get(
-        HAS_TPU_PROVIDER_FIELD, False) else None
+    tpu_resource = (
+        _create_tpu(credentials)
+        if provider_config.get(HAS_TPU_PROVIDER_FIELD, False)
+        else None
+    )
 
-    return _create_crm(credentials), \
-        _create_iam(credentials), \
-        _create_compute(credentials), \
-        tpu_resource
+    return (
+        _create_crm(credentials),
+        _create_iam(credentials),
+        _create_compute(credentials),
+        tpu_resource,
+    )
 
 
 def bootstrap_gcp(config):
@@ -265,8 +291,7 @@ def bootstrap_gcp(config):
         if _is_head_node_a_tpu(config):
             raise RuntimeError("TPUs are not supported as head nodes.")
 
-    crm, iam, compute, tpu = \
-        construct_clients_from_provider_config(config["provider"])
+    crm, iam, compute, tpu = construct_clients_from_provider_config(config["provider"])
 
     config = _configure_project(config, crm)
     config = _configure_iam_role(config, crm, iam)
@@ -288,7 +313,8 @@ def _configure_project(config, crm):
     project_id = config["provider"].get("project_id")
     assert config["provider"]["project_id"] is not None, (
         "'project_id' must be set in the 'provider' section of the autoscaler"
-        " config. Notice that the project id must be globally unique.")
+        " config. Notice that the project id must be globally unique."
+    )
     project = _get_project(project_id, crm)
 
     if project is None:
@@ -297,9 +323,9 @@ def _configure_project(config, crm):
         project = _get_project(project_id, crm)
 
     assert project is not None, "Failed to create project"
-    assert project["lifecycleState"] == "ACTIVE", (
-        "Project status needs to be ACTIVE, got {}".format(
-            project["lifecycleState"]))
+    assert (
+        project["lifecycleState"] == "ACTIVE"
+    ), "Project status needs to be ACTIVE, got {}".format(project["lifecycleState"])
 
     config["provider"]["project_id"] = project["projectId"]
 
@@ -320,17 +346,19 @@ def _configure_iam_role(config, crm, iam):
 
     email = SERVICE_ACCOUNT_EMAIL_TEMPLATE.format(
         account_id=DEFAULT_SERVICE_ACCOUNT_ID,
-        project_id=config["provider"]["project_id"])
+        project_id=config["provider"]["project_id"],
+    )
     service_account = _get_service_account(email, config, iam)
 
     if service_account is None:
-        logger.info("_configure_iam_role: "
-                    "Creating new service account {}".format(
-                        DEFAULT_SERVICE_ACCOUNT_ID))
+        logger.info(
+            "_configure_iam_role: "
+            "Creating new service account {}".format(DEFAULT_SERVICE_ACCOUNT_ID)
+        )
 
         service_account = _create_service_account(
-            DEFAULT_SERVICE_ACCOUNT_ID, DEFAULT_SERVICE_ACCOUNT_CONFIG, config,
-            iam)
+            DEFAULT_SERVICE_ACCOUNT_ID, DEFAULT_SERVICE_ACCOUNT_CONFIG, config, iam
+        )
 
     assert service_account is not None, "Failed to create service account"
 
@@ -341,14 +369,16 @@ def _configure_iam_role(config, crm, iam):
 
     _add_iam_policy_binding(service_account, roles, crm)
 
-    config["head_node"]["serviceAccounts"] = [{
-        "email": service_account["email"],
-        # NOTE: The amount of access is determined by the scope + IAM
-        # role of the service account. Even if the cloud-platform scope
-        # gives (scope) access to the whole cloud-platform, the service
-        # account is limited by the IAM rights specified below.
-        "scopes": ["https://www.googleapis.com/auth/cloud-platform"]
-    }]
+    config["head_node"]["serviceAccounts"] = [
+        {
+            "email": service_account["email"],
+            # NOTE: The amount of access is determined by the scope + IAM
+            # role of the service account. Even if the cloud-platform scope
+            # gives (scope) access to the whole cloud-platform, the service
+            # account is limited by the IAM rights specified below.
+            "scopes": ["https://www.googleapis.com/auth/cloud-platform"],
+        }
+    ]
 
     return config
 
@@ -375,22 +405,27 @@ def _configure_key_pair(config, compute):
 
     ssh_user = config["auth"]["ssh_user"]
 
-    project = compute.projects().get(
-        project=config["provider"]["project_id"]).execute()
+    project = compute.projects().get(project=config["provider"]["project_id"]).execute()
 
     # Key pairs associated with project meta data. The key pairs are general,
     # and not just ssh keys.
     ssh_keys_str = next(
-        (item for item in project["commonInstanceMetadata"].get("items", [])
-         if item["key"] == "ssh-keys"), {}).get("value", "")
+        (
+            item
+            for item in project["commonInstanceMetadata"].get("items", [])
+            if item["key"] == "ssh-keys"
+        ),
+        {},
+    ).get("value", "")
 
     ssh_keys = ssh_keys_str.split("\n") if ssh_keys_str else []
 
     # Try a few times to get or create a good key pair.
     key_found = False
     for i in range(10):
-        key_name = key_pair_name(i, config["provider"]["region"],
-                                 config["provider"]["project_id"], ssh_user)
+        key_name = key_pair_name(
+            i, config["provider"]["region"], config["provider"]["project_id"], ssh_user
+        )
         public_key_path, private_key_path = key_pair_paths(key_name)
 
         for ssh_key in ssh_keys:
@@ -409,12 +444,12 @@ def _configure_key_pair(config, compute):
 
         # Create a key since it doesn't exist locally or in GCP
         if not key_found and not os.path.exists(private_key_path):
-            logger.info("_configure_key_pair: "
-                        "Creating new key pair {}".format(key_name))
+            logger.info(
+                "_configure_key_pair: Creating new key pair {}".format(key_name)
+            )
             public_key, private_key = generate_rsa_key_pair()
 
-            _create_project_ssh_key_pair(project, public_key, ssh_user,
-                                         compute)
+            _create_project_ssh_key_pair(project, public_key, ssh_user, compute)
 
             # Create the directory if it doesn't exists
             private_key_dir = os.path.dirname(private_key_path)
@@ -424,9 +459,9 @@ def _configure_key_pair(config, compute):
             # permissions. In order to do that we need to change the default
             # os.open behavior to include the mode we want.
             with open(
-                    private_key_path,
-                    "w",
-                    opener=partial(os.open, mode=0o600),
+                private_key_path,
+                "w",
+                opener=partial(os.open, mode=0o600),
             ) as f:
                 f.write(private_key)
 
@@ -441,14 +476,17 @@ def _configure_key_pair(config, compute):
             break
 
     assert key_found, "SSH keypair for user {} not found for {}".format(
-        ssh_user, private_key_path)
-    assert os.path.exists(private_key_path), (
-        "Private key file {} not found for user {}"
-        "".format(private_key_path, ssh_user))
+        ssh_user, private_key_path
+    )
+    assert os.path.exists(
+        private_key_path
+    ), "Private key file {} not found for user {}".format(private_key_path, ssh_user)
 
-    logger.info("_configure_key_pair: "
-                "Private key not specified in config, using"
-                "{}".format(private_key_path))
+    logger.info(
+        "_configure_key_pair: "
+        "Private key not specified in config, using"
+        "{}".format(private_key_path)
+    )
 
     config["auth"]["ssh_private_key"] = private_key_path
 
@@ -467,8 +505,10 @@ def _configure_subnet(config, compute):
     # completely manually configured
 
     # networkInterfaces is compute, networkConfig is TPU
-    if all("networkInterfaces" in node_config or "networkConfig" in node_config
-           for node_config in node_configs):
+    if all(
+        "networkInterfaces" in node_config or "networkConfig" in node_config
+        for node_config in node_configs
+    ):
         return config
 
     subnets = _list_subnets(config, compute)
@@ -481,21 +521,24 @@ def _configure_subnet(config, compute):
     # work out-of-the-box
     default_subnet = subnets[0]
 
-    default_interfaces = [{
-        "subnetwork": default_subnet["selfLink"],
-        "accessConfigs": [{
-            "name": "External NAT",
-            "type": "ONE_TO_ONE_NAT",
-        }],
-    }]
+    default_interfaces = [
+        {
+            "subnetwork": default_subnet["selfLink"],
+            "accessConfigs": [
+                {
+                    "name": "External NAT",
+                    "type": "ONE_TO_ONE_NAT",
+                }
+            ],
+        }
+    ]
 
     for node_config in node_configs:
         # The not applicable key will be removed during node creation
 
         # compute
         if "networkInterfaces" not in node_config:
-            node_config["networkInterfaces"] = copy.deepcopy(
-                default_interfaces)
+            node_config["networkInterfaces"] = copy.deepcopy(default_interfaces)
         # TPU
         if "networkConfig" not in node_config:
             node_config["networkConfig"] = copy.deepcopy(default_interfaces)[0]
@@ -505,19 +548,28 @@ def _configure_subnet(config, compute):
 
 
 def _list_subnets(config, compute):
-    response = compute.subnetworks().list(
-        project=config["provider"]["project_id"],
-        region=config["provider"]["region"]).execute()
+    response = (
+        compute.subnetworks()
+        .list(
+            project=config["provider"]["project_id"],
+            region=config["provider"]["region"],
+        )
+        .execute()
+    )
 
     return response["items"]
 
 
 def _get_subnet(config, subnet_id, compute):
-    subnet = compute.subnetworks().get(
-        project=config["provider"]["project_id"],
-        region=config["provider"]["region"],
-        subnetwork=subnet_id,
-    ).execute()
+    subnet = (
+        compute.subnetworks()
+        .get(
+            project=config["provider"]["project_id"],
+            region=config["provider"]["region"],
+            subnetwork=subnet_id,
+        )
+        .execute()
+    )
 
     return subnet
 
@@ -534,10 +586,11 @@ def _get_project(project_id, crm):
 
 
 def _create_project(project_id, crm):
-    operation = crm.projects().create(body={
-        "projectId": project_id,
-        "name": project_id
-    }).execute()
+    operation = (
+        crm.projects()
+        .create(body={"projectId": project_id, "name": project_id})
+        .execute()
+    )
 
     result = wait_for_crm_operation(operation, crm)
 
@@ -546,11 +599,11 @@ def _create_project(project_id, crm):
 
 def _get_service_account(account, config, iam):
     project_id = config["provider"]["project_id"]
-    full_name = ("projects/{project_id}/serviceAccounts/{account}"
-                 "".format(project_id=project_id, account=account))
+    full_name = "projects/{project_id}/serviceAccounts/{account}".format(
+        project_id=project_id, account=account
+    )
     try:
-        service_account = iam.projects().serviceAccounts().get(
-            name=full_name).execute()
+        service_account = iam.projects().serviceAccounts().get(name=full_name).execute()
     except errors.HttpError as e:
         if e.resp.status != 404:
             raise
@@ -562,12 +615,18 @@ def _get_service_account(account, config, iam):
 def _create_service_account(account_id, account_config, config, iam):
     project_id = config["provider"]["project_id"]
 
-    service_account = iam.projects().serviceAccounts().create(
-        name="projects/{project_id}".format(project_id=project_id),
-        body={
-            "accountId": account_id,
-            "serviceAccount": account_config,
-        }).execute()
+    service_account = (
+        iam.projects()
+        .serviceAccounts()
+        .create(
+            name="projects/{project_id}".format(project_id=project_id),
+            body={
+                "accountId": account_id,
+                "serviceAccount": account_config,
+            },
+        )
+        .execute()
+    )
 
     return service_account
 
@@ -578,8 +637,7 @@ def _add_iam_policy_binding(service_account, roles, crm):
     email = service_account["email"]
     member_id = "serviceAccount:" + email
 
-    policy = crm.projects().getIamPolicy(
-        resource=project_id, body={}).execute()
+    policy = crm.projects().getIamPolicy(resource=project_id, body={}).execute()
 
     already_configured = True
     for role in roles:
@@ -593,20 +651,28 @@ def _add_iam_policy_binding(service_account, roles, crm):
 
         if not role_exists:
             already_configured = False
-            policy["bindings"].append({
-                "members": [member_id],
-                "role": role,
-            })
+            policy["bindings"].append(
+                {
+                    "members": [member_id],
+                    "role": role,
+                }
+            )
 
     if already_configured:
         # In some managed environments, an admin needs to grant the
         # roles, so only call setIamPolicy if needed.
         return
 
-    result = crm.projects().setIamPolicy(
-        resource=project_id, body={
-            "policy": policy,
-        }).execute()
+    result = (
+        crm.projects()
+        .setIamPolicy(
+            resource=project_id,
+            body={
+                "policy": policy,
+            },
+        )
+        .execute()
+    )
 
     return result
 
@@ -621,13 +687,15 @@ def _create_project_ssh_key_pair(project, public_key, ssh_user, compute):
     assert key_parts[0] == "ssh-rsa", key_parts
 
     new_ssh_meta = "{ssh_user}:ssh-rsa {key_value} {ssh_user}".format(
-        ssh_user=ssh_user, key_value=key_parts[1])
+        ssh_user=ssh_user, key_value=key_parts[1]
+    )
 
     common_instance_metadata = project["commonInstanceMetadata"]
     items = common_instance_metadata.get("items", [])
 
     ssh_keys_i = next(
-        (i for i, item in enumerate(items) if item["key"] == "ssh-keys"), None)
+        (i for i, item in enumerate(items) if item["key"] == "ssh-keys"), None
+    )
 
     if ssh_keys_i is None:
         items.append({"key": "ssh-keys", "value": new_ssh_meta})
@@ -638,10 +706,14 @@ def _create_project_ssh_key_pair(project, public_key, ssh_user, compute):
 
     common_instance_metadata["items"] = items
 
-    operation = compute.projects().setCommonInstanceMetadata(
-        project=project["name"], body=common_instance_metadata).execute()
+    operation = (
+        compute.projects()
+        .setCommonInstanceMetadata(
+            project=project["name"], body=common_instance_metadata
+        )
+        .execute()
+    )
 
-    response = wait_for_compute_global_operation(project["name"], operation,
-                                                 compute)
+    response = wait_for_compute_global_operation(project["name"], operation, compute)
 
     return response
