@@ -44,7 +44,8 @@ def create_parser(parser_creator=None):
         parser_creator=parser_creator,
         formatter_class=argparse.RawDescriptionHelpFormatter,
         description="Train a reinforcement learning agent.",
-        epilog=EXAMPLE_USAGE)
+        epilog=EXAMPLE_USAGE,
+    )
 
     # See also the base parser definition in ray/tune/config_parser.py
     parser.add_argument(
@@ -52,94 +53,113 @@ def create_parser(parser_creator=None):
         default=None,
         type=str,
         help="Connect to an existing Ray cluster at this address instead "
-        "of starting a new one.")
+        "of starting a new one.",
+    )
     parser.add_argument(
-        "--ray-ui",
-        action="store_true",
-        help="Whether to enable the Ray web UI.")
+        "--ray-ui", action="store_true", help="Whether to enable the Ray web UI."
+    )
     # Deprecated: Use --ray-ui, instead.
     parser.add_argument(
         "--no-ray-ui",
         action="store_true",
         help="Deprecated! Ray UI is disabled by default now. "
-        "Use `--ray-ui` to enable.")
+        "Use `--ray-ui` to enable.",
+    )
     parser.add_argument(
         "--local-mode",
         action="store_true",
-        help="Run ray in local mode for easier debugging.")
+        help="Run ray in local mode for easier debugging.",
+    )
     parser.add_argument(
         "--ray-num-cpus",
         default=None,
         type=int,
-        help="--num-cpus to use if starting a new cluster.")
+        help="--num-cpus to use if starting a new cluster.",
+    )
     parser.add_argument(
         "--ray-num-gpus",
         default=None,
         type=int,
-        help="--num-gpus to use if starting a new cluster.")
+        help="--num-gpus to use if starting a new cluster.",
+    )
     parser.add_argument(
         "--ray-num-nodes",
         default=None,
         type=int,
-        help="Emulate multiple cluster nodes for debugging.")
+        help="Emulate multiple cluster nodes for debugging.",
+    )
     parser.add_argument(
         "--ray-object-store-memory",
         default=None,
         type=int,
-        help="--object-store-memory to use if starting a new cluster.")
+        help="--object-store-memory to use if starting a new cluster.",
+    )
     parser.add_argument(
         "--experiment-name",
         default="default",
         type=str,
-        help="Name of the subdirectory under `local_dir` to put results in.")
+        help="Name of the subdirectory under `local_dir` to put results in.",
+    )
     parser.add_argument(
         "--local-dir",
         default=DEFAULT_RESULTS_DIR,
         type=str,
         help="Local dir to save training results to. Defaults to '{}'.".format(
-            DEFAULT_RESULTS_DIR))
+            DEFAULT_RESULTS_DIR
+        ),
+    )
     parser.add_argument(
         "--upload-dir",
         default="",
         type=str,
-        help="Optional URI to sync training results to (e.g. s3://bucket).")
+        help="Optional URI to sync training results to (e.g. s3://bucket).",
+    )
     # This will override any framework setting found in a yaml file.
     parser.add_argument(
         "--framework",
         choices=["tf", "tf2", "tfe", "torch"],
         default=None,
-        help="The DL framework specifier.")
+        help="The DL framework specifier.",
+    )
     parser.add_argument(
-        "-v", action="store_true", help="Whether to use INFO level logging.")
+        "-v", action="store_true", help="Whether to use INFO level logging."
+    )
     parser.add_argument(
-        "-vv", action="store_true", help="Whether to use DEBUG level logging.")
+        "-vv", action="store_true", help="Whether to use DEBUG level logging."
+    )
     parser.add_argument(
         "--resume",
         action="store_true",
-        help="Whether to attempt to resume previous Tune experiments.")
+        help="Whether to attempt to resume previous Tune experiments.",
+    )
     parser.add_argument(
         "--trace",
         action="store_true",
-        help="Whether to attempt to enable tracing for eager mode.")
+        help="Whether to attempt to enable tracing for eager mode.",
+    )
     parser.add_argument(
-        "--env", default=None, type=str, help="The gym environment to use.")
+        "--env", default=None, type=str, help="The gym environment to use."
+    )
     parser.add_argument(
         "-f",
         "--config-file",
         default=None,
         type=str,
         help="If specified, use config options from this file. Note that this "
-        "overrides any trial-specific options set via flags above.")
+        "overrides any trial-specific options set via flags above.",
+    )
 
     # Obsolete: Use --framework=torch|tf2|tfe instead!
     parser.add_argument(
         "--torch",
         action="store_true",
-        help="Whether to use PyTorch (instead of tf) as the DL framework.")
+        help="Whether to use PyTorch (instead of tf) as the DL framework.",
+    )
     parser.add_argument(
         "--eager",
         action="store_true",
-        help="Whether to attempt to enable TF eager execution.")
+        help="Whether to attempt to enable TF eager execution.",
+    )
 
     return parser
 
@@ -158,16 +178,17 @@ def run(args, parser):
                 "keep_checkpoints_num": args.keep_checkpoints_num,
                 "checkpoint_score_attr": args.checkpoint_score_attr,
                 "local_dir": args.local_dir,
-                "resources_per_trial": (args.resources_per_trial
-                                        and resources_to_json(
-                                            args.resources_per_trial)),
+                "resources_per_trial": (
+                    args.resources_per_trial
+                    and resources_to_json(args.resources_per_trial)
+                ),
                 "stop": args.stop,
                 "config": dict(args.config, env=args.env),
                 "restore": args.restore,
                 "num_samples": args.num_samples,
                 "sync_config": {
                     "upload_dir": args.upload_dir,
-                }
+                },
             }
         }
 
@@ -191,10 +212,7 @@ def run(args, parser):
                 if isinstance(path, list):
                     return [patch_path(i) for i in path]
                 elif isinstance(path, dict):
-                    return {
-                        patch_path(k): patch_path(v)
-                        for k, v in path.items()
-                    }
+                    return {patch_path(k): patch_path(v) for k, v in path.items()}
                 elif isinstance(path, str):
                     if os.path.exists(path):
                         return path
@@ -236,12 +254,14 @@ def run(args, parser):
         # Import this only here so that train.py also works with
         # older versions (and user doesn't use `--ray-num-nodes`).
         from ray.cluster_utils import Cluster
+
         cluster = Cluster()
         for _ in range(args.ray_num_nodes):
             cluster.add_node(
                 num_cpus=args.ray_num_cpus or 1,
                 num_gpus=args.ray_num_gpus or 0,
-                object_store_memory=args.ray_object_store_memory)
+                object_store_memory=args.ray_object_store_memory,
+            )
         ray.init(address=cluster.address)
     else:
         ray.init(
@@ -250,11 +270,13 @@ def run(args, parser):
             object_store_memory=args.ray_object_store_memory,
             num_cpus=args.ray_num_cpus,
             num_gpus=args.ray_num_gpus,
-            local_mode=args.local_mode)
+            local_mode=args.local_mode,
+        )
 
     if IS_NOTEBOOK:
         progress_reporter = JupyterNotebookReporter(
-            overwrite=verbose >= 3, print_intermediate_tables=verbose >= 1)
+            overwrite=verbose >= 3, print_intermediate_tables=verbose >= 1
+        )
     else:
         progress_reporter = CLIReporter(print_intermediate_tables=verbose >= 1)
 
@@ -264,7 +286,8 @@ def run(args, parser):
         resume=args.resume,
         verbose=verbose,
         progress_reporter=progress_reporter,
-        concurrent=True)
+        concurrent=True,
+    )
 
     ray.shutdown()
 
