@@ -13,6 +13,7 @@ from ray.rllib.utils.typing import LocalOptimizer, TrainerConfigDict
 if TYPE_CHECKING:
     from ray.rllib.policy.policy import Policy
     from ray.rllib.utils import try_import_tf
+
     _, tf, _ = try_import_tf()
 
 _, nn = try_import_torch()
@@ -27,9 +28,16 @@ class Exploration:
     implemented exploration schema.
     """
 
-    def __init__(self, action_space: Space, *, framework: str,
-                 policy_config: TrainerConfigDict, model: ModelV2,
-                 num_workers: int, worker_index: int):
+    def __init__(
+        self,
+        action_space: Space,
+        *,
+        framework: str,
+        policy_config: TrainerConfigDict,
+        model: ModelV2,
+        num_workers: int,
+        worker_index: int
+    ):
         """
         Args:
             action_space: The action space in which to explore.
@@ -55,12 +63,13 @@ class Exploration:
 
     @DeveloperAPI
     def before_compute_actions(
-            self,
-            *,
-            timestep: Optional[Union[TensorType, int]] = None,
-            explore: Optional[Union[TensorType, bool]] = None,
-            tf_sess: Optional["tf.Session"] = None,
-            **kwargs):
+        self,
+        *,
+        timestep: Optional[Union[TensorType, int]] = None,
+        explore: Optional[Union[TensorType, bool]] = None,
+        tf_sess: Optional["tf.Session"] = None,
+        **kwargs
+    ):
         """Hook for preparations before policy.compute_actions() is called.
 
         Args:
@@ -71,7 +80,7 @@ class Exploration:
         """
         pass
 
-    # yapf: disable
+    # fmt: off
     # __sphinx_doc_begin_get_exploration_action__
 
     @DeveloperAPI
@@ -103,15 +112,17 @@ class Exploration:
         pass
 
     # __sphinx_doc_end_get_exploration_action__
-    # yapf: enable
+    # fmt: on
 
     @DeveloperAPI
-    def on_episode_start(self,
-                         policy: "Policy",
-                         *,
-                         environment: BaseEnv = None,
-                         episode: int = None,
-                         tf_sess: Optional["tf.Session"] = None):
+    def on_episode_start(
+        self,
+        policy: "Policy",
+        *,
+        environment: BaseEnv = None,
+        episode: int = None,
+        tf_sess: Optional["tf.Session"] = None
+    ):
         """Handles necessary exploration logic at the beginning of an episode.
 
         Args:
@@ -123,12 +134,14 @@ class Exploration:
         pass
 
     @DeveloperAPI
-    def on_episode_end(self,
-                       policy: "Policy",
-                       *,
-                       environment: BaseEnv = None,
-                       episode: int = None,
-                       tf_sess: Optional["tf.Session"] = None):
+    def on_episode_end(
+        self,
+        policy: "Policy",
+        *,
+        environment: BaseEnv = None,
+        episode: int = None,
+        tf_sess: Optional["tf.Session"] = None
+    ):
         """Handles necessary exploration logic at the end of an episode.
 
         Args:
@@ -140,10 +153,12 @@ class Exploration:
         pass
 
     @DeveloperAPI
-    def postprocess_trajectory(self,
-                               policy: "Policy",
-                               sample_batch: SampleBatch,
-                               tf_sess: Optional["tf.Session"] = None):
+    def postprocess_trajectory(
+        self,
+        policy: "Policy",
+        sample_batch: SampleBatch,
+        tf_sess: Optional["tf.Session"] = None,
+    ):
         """Handles post-processing of done episode trajectories.
 
         Changes the given batch in place. This callback is invoked by the
@@ -157,8 +172,9 @@ class Exploration:
         return sample_batch
 
     @DeveloperAPI
-    def get_exploration_optimizer(self, optimizers: List[LocalOptimizer]) -> \
-            List[LocalOptimizer]:
+    def get_exploration_optimizer(
+        self, optimizers: List[LocalOptimizer]
+    ) -> List[LocalOptimizer]:
         """May add optimizer(s) to the Policy's own `optimizers`.
 
         The number of optimizers (Policy's plus Exploration's optimizers) must
@@ -175,8 +191,7 @@ class Exploration:
         return optimizers
 
     @DeveloperAPI
-    def get_state(self, sess: Optional["tf.Session"] = None) -> \
-            Dict[str, TensorType]:
+    def get_state(self, sess: Optional["tf.Session"] = None) -> Dict[str, TensorType]:
         """Returns the current exploration state.
 
         Args:
@@ -188,8 +203,7 @@ class Exploration:
         return {}
 
     @DeveloperAPI
-    def set_state(self, state: object,
-                  sess: Optional["tf.Session"] = None) -> None:
+    def set_state(self, state: object, sess: Optional["tf.Session"] = None) -> None:
         """Sets the Exploration object's state to the given values.
 
         Note that some exploration components are stateless, even though they
