@@ -18,12 +18,14 @@ from ray import serve
 from ray.cluster_utils import Cluster, cluster_not_supported
 from ray.serve.constants import SERVE_ROOT_URL_ENV_KEY, SERVE_PROXY_NAME
 from ray.serve.exceptions import RayServeException
-from ray.serve.utils import (block_until_http_ready, get_all_node_ids,
-                             format_actor_name)
+from ray.serve.utils import block_until_http_ready, get_all_node_ids, format_actor_name
 from ray.serve.config import HTTPOptions
-from ray.serve.api import _get_global_client
-from ray._private.test_utils import (run_string_as_driver, wait_for_condition,
-                                     convert_actor_state)
+from ray.serve.api import internal_get_global_client
+from ray._private.test_utils import (
+    run_string_as_driver,
+    wait_for_condition,
+    convert_actor_state,
+)
 from ray._private.services import new_port
 import ray._private.gcs_utils as gcs_utils
 
@@ -56,9 +58,11 @@ def test_shutdown(ray_shutdown):
     serve_controller_name = serve.api._global_client._controller_name
     actor_names = [
         serve_controller_name,
-        format_actor_name(SERVE_PROXY_NAME,
-                          serve.api._global_client._controller_name,
-                          get_all_node_ids()[0][0])
+        format_actor_name(
+            SERVE_PROXY_NAME,
+            serve.api._global_client._controller_name,
+            get_all_node_ids()[0][0],
+        ),
     ]
 
     def check_alive():
@@ -67,8 +71,8 @@ def test_shutdown(ray_shutdown):
             try:
                 if actor_name == serve_controller_name:
                     ray.get_actor(
-                        actor_name,
-                        namespace=ray.get_runtime_context().namespace)
+                        actor_name, namespace=ray.get_runtime_context().namespace
+                    )
                 else:
                     ray.get_actor(actor_name)
             except ValueError:
@@ -86,8 +90,8 @@ def test_shutdown(ray_shutdown):
             try:
                 if actor_name == serve_controller_name:
                     ray.get_actor(
-                        actor_name,
-                        namespace=ray.get_runtime_context().namespace)
+                        actor_name, namespace=ray.get_runtime_context().namespace
+                    )
                 else:
                     ray.get_actor(actor_name)
                 return False
@@ -158,25 +162,26 @@ def test_dedicated_cpu(controller_cpu, num_proxy_cpus, ray_cluster):
     head_node = cluster.add_node(num_cpus=num_cluster_cpus)
 
     ray.init(head_node.address)
-    wait_for_condition(
-        lambda: ray.cluster_resources().get("CPU") == num_cluster_cpus)
+    wait_for_condition(lambda: ray.cluster_resources().get("CPU") == num_cluster_cpus)
 
     num_cpus_used = int(controller_cpu) + num_proxy_cpus
 
     serve.start(
-        dedicated_cpu=controller_cpu,
-        http_options=HTTPOptions(num_cpus=num_proxy_cpus))
+        dedicated_cpu=controller_cpu, http_options=HTTPOptions(num_cpus=num_proxy_cpus)
+    )
     available_cpus = num_cluster_cpus - num_cpus_used
-    wait_for_condition(
-        lambda: (ray.available_resources().get("CPU") == available_cpus))
+    wait_for_condition(lambda: (ray.available_resources().get("CPU") == available_cpus))
     serve.shutdown()
     ray.shutdown()
 
 
 @pytest.mark.skipif(
     not hasattr(socket, "SO_REUSEPORT"),
-    reason=("Port sharing only works on newer verion of Linux. "
-            "This test can only be ran when port sharing is supported."))
+    reason=(
+        "Port sharing only works on newer verion of Linux. "
+        "This test can only be ran when port sharing is supported."
+    ),
+)
 def test_multiple_routers(ray_cluster):
     cluster = ray_cluster
     head_node = cluster.add_node(num_cpus=4)
@@ -191,9 +196,10 @@ def test_multiple_routers(ray_cluster):
         proxy_names = []
         for node_id, _ in get_all_node_ids():
             proxy_names.append(
-                format_actor_name(SERVE_PROXY_NAME,
-                                  serve.api._global_client._controller_name,
-                                  node_id))
+                format_actor_name(
+                    SERVE_PROXY_NAME, serve.api._global_client._controller_name, node_id
+                )
+            )
         return proxy_names
 
     wait_for_condition(lambda: len(get_proxy_names()) == 2)
@@ -259,9 +265,10 @@ def test_middleware(ray_shutdown):
         http_options=dict(
             port=port,
             middlewares=[
-                Middleware(
-                    CORSMiddleware, allow_origins=["*"], allow_methods=["*"])
-            ]))
+                Middleware(CORSMiddleware, allow_origins=["*"], allow_methods=["*"])
+            ],
+        )
+    )
     ray.get(block_until_http_ready.remote(f"http://127.0.0.1:{port}/-/routes"))
 
     # Snatched several test cases from Starlette
@@ -313,6 +320,32 @@ def test_http_root_url(ray_shutdown):
     ray.shutdown()
 
 
+@pytest.mark.skipif(sys.platform == "win32", reason="Failing on Windows")
+def test_http_root_path(ray_shutdown):
+    @serve.deployment
+    def hello():
+        return "hello"
+
+    port = new_port()
+    root_path = "/serve"
+    serve.start(http_options=dict(root_path=root_path, port=port))
+    hello.deploy()
+
+    # check whether url is prefixed correctly
+    assert hello.url == f"http://127.0.0.1:{port}{root_path}/hello"
+
+    # check routing works as expected
+    resp = requests.get(hello.url)
+    assert resp.status_code == 200
+    assert resp.text == "hello"
+
+    # check advertized routes are prefixed correctly
+    resp = requests.get(f"http://127.0.0.1:{port}{root_path}/-/routes")
+    assert resp.status_code == 200
+    assert resp.json() == {"/hello": "hello"}
+
+
+@pytest.mark.skipif(sys.platform == "win32", reason="Failing on Windows")
 def test_http_proxy_fail_loudly(ray_shutdown):
     # Test that if the http server fail to start, serve.start should fail.
     with pytest.raises(ValueError):
@@ -322,21 +355,9 @@ def test_http_proxy_fail_loudly(ray_shutdown):
 def test_no_http(ray_shutdown):
     # The following should have the same effect.
     options = [
-        {
-            "http_options": {
-                "host": None
-            }
-        },
-        {
-            "http_options": {
-                "location": None
-            }
-        },
-        {
-            "http_options": {
-                "location": "NoServer"
-            }
-        },
+        {"http_options": {"host": None}},
+        {"http_options": {"location": None}},
+        {"http_options": {"location": "NoServer"}},
     ]
 
     ray.init(num_cpus=16)
@@ -346,8 +367,9 @@ def test_no_http(ray_shutdown):
 
         # Only controller actor should exist
         live_actors = [
-            actor for actor in ray.state.actors().values() if actor["State"] ==
-            convert_actor_state(gcs_utils.ActorTableData.ALIVE)
+            actor
+            for actor in ray.state.actors().values()
+            if actor["State"] == convert_actor_state(gcs_utils.ActorTableData.ALIVE)
         ]
         assert len(live_actors) == 1
         controller = serve.api._global_client._controller
@@ -380,16 +402,18 @@ def test_http_head_only(ray_cluster):
 
     # They should all be placed on the head node
     cpu_per_nodes = {
-        r["CPU"]
-        for r in ray.state.state._available_resources_per_node().values()
+        r["CPU"] for r in ray.state.state._available_resources_per_node().values()
     }
     assert cpu_per_nodes == {4, 4}
 
 
 @pytest.mark.skipif(
     not hasattr(socket, "SO_REUSEPORT"),
-    reason=("Port sharing only works on newer verion of Linux. "
-            "This test can only be ran when port sharing is supported."))
+    reason=(
+        "Port sharing only works on newer verion of Linux. "
+        "This test can only be ran when port sharing is supported."
+    ),
+)
 def test_fixed_number_proxies(ray_cluster):
     cluster = ray_cluster
     head_node = cluster.add_node(num_cpus=4)
@@ -401,20 +425,25 @@ def test_fixed_number_proxies(ray_cluster):
     assert len(node_ids) == 3
 
     with pytest.raises(
-            pydantic.ValidationError,
-            match="you must specify the `fixed_number_replicas` parameter."):
-        serve.start(http_options={
-            "location": "FixedNumber",
-        })
+        pydantic.ValidationError,
+        match="you must specify the `fixed_number_replicas` parameter.",
+    ):
+        serve.start(
+            http_options={
+                "location": "FixedNumber",
+            }
+        )
 
-    serve.start(http_options={
-        "port": new_port(),
-        "location": "FixedNumber",
-        "fixed_number_replicas": 2
-    })
+    serve.start(
+        http_options={
+            "port": new_port(),
+            "location": "FixedNumber",
+            "fixed_number_replicas": 2,
+        }
+    )
 
     # Only the controller and two http proxy should be started.
-    controller_handle = _get_global_client()._controller
+    controller_handle = internal_get_global_client()._controller
     node_to_http_actors = ray.get(controller_handle.get_http_proxies.remote())
     assert len(node_to_http_actors) == 2
 
@@ -459,8 +488,9 @@ def test_detached_instance_in_non_anonymous_namespace(ray_shutdown):
 
 @pytest.mark.parametrize("namespace", [None, "test_namespace"])
 @pytest.mark.parametrize("detached", [True, False])
-def test_serve_controller_namespace(ray_shutdown, namespace: Optional[str],
-                                    detached: bool):
+def test_serve_controller_namespace(
+    ray_shutdown, namespace: Optional[str], detached: bool
+):
     """
     Tests the serve controller is started in the current namespace if not
     anonymous or in the "serve" namespace if no namespace is specified.
@@ -478,8 +508,7 @@ def test_serve_controller_namespace(ray_shutdown, namespace: Optional[str],
     else:
         controller_namespace = ray.get_runtime_context().namespace
 
-    assert ray.get_actor(
-        client._controller_name, namespace=controller_namespace)
+    assert ray.get_actor(client._controller_name, namespace=controller_namespace)
 
 
 def test_checkpoint_isolation_namespace(ray_shutdown):
@@ -502,11 +531,11 @@ class A:
 A.deploy()"""
 
     run_string_as_driver(
-        driver_template.format(
-            address=address, namespace="test_namespace1", port=8000))
+        driver_template.format(address=address, namespace="test_namespace1", port=8000)
+    )
     run_string_as_driver(
-        driver_template.format(
-            address=address, namespace="test_namespace2", port=8001))
+        driver_template.format(address=address, namespace="test_namespace2", port=8001)
+    )
 
 
 def test_local_store_recovery(ray_shutdown):
@@ -533,6 +562,7 @@ def test_local_store_recovery(ray_shutdown):
     # https://github.com/ray-project/ray/issues/20158
     def clean_up_leaked_processes():
         import psutil
+
         for proc in psutil.process_iter():
             try:
                 cmdline = " ".join(proc.cmdline())
@@ -563,13 +593,10 @@ def test_local_store_recovery(ray_shutdown):
     crash()
 
 
-@pytest.mark.parametrize(
-    "ray_start_with_dashboard", [{
-        "num_cpus": 4
-    }], indirect=True)
+@pytest.mark.parametrize("ray_start_with_dashboard", [{"num_cpus": 4}], indirect=True)
 def test_snapshot_always_written_to_internal_kv(
-        ray_start_with_dashboard,  # noqa: F811
-        ray_shutdown):
+    ray_start_with_dashboard, ray_shutdown  # noqa: F811
+):
     # https://github.com/ray-project/ray/issues/19752
     _, tmp_path = mkstemp()
 
@@ -592,8 +619,9 @@ def test_snapshot_always_written_to_internal_kv(
     webui_url = ray_start_with_dashboard["webui_url"]
 
     def get_deployment_snapshot():
-        snapshot = requests.get(f"http://{webui_url}/api/snapshot").json()[
-            "data"]["snapshot"]
+        snapshot = requests.get(f"http://{webui_url}/api/snapshot").json()["data"][
+            "snapshot"
+        ]
         return snapshot["deployments"]
 
     # Make sure /api/snapshot return non-empty deployment status.
@@ -634,8 +662,7 @@ def test_serve_start_different_http_checkpoint_options_warning(caplog):
     _, tmp_path = mkstemp()
     test_ckpt = f"file://{tmp_path}"
 
-    serve.start(
-        detached=True, http_options=test_http, _checkpoint_path=test_ckpt)
+    serve.start(detached=True, http_options=test_http, _checkpoint_path=test_ckpt)
 
     for test_config, msg in zip([[test_ckpt], ["host", "port"]], warning_msg):
         for test_msg in test_config:
