@@ -21,11 +21,12 @@ def merge_dicts(d1: dict, d2: dict) -> dict:
 
 
 def deep_update(
-        original: dict,
-        new_dict: dict,
-        new_keys_allowed: bool = False,
-        allow_new_subkey_list: Optional[List[str]] = None,
-        override_all_if_type_changes: Optional[List[str]] = None) -> dict:
+    original: dict,
+    new_dict: dict,
+    new_keys_allowed: bool = False,
+    allow_new_subkey_list: Optional[List[str]] = None,
+    override_all_if_type_changes: Optional[List[str]] = None,
+) -> dict:
     """Updates original dict with values from new_dict recursively.
 
     If new key is introduced in new_dict, then if new_keys_allowed is not
@@ -53,9 +54,12 @@ def deep_update(
         # Both orginal value and new one are dicts.
         if isinstance(original.get(k), dict) and isinstance(value, dict):
             # Check old type vs old one. If different, override entire value.
-            if k in override_all_if_type_changes and \
-                "type" in value and "type" in original[k] and \
-                    value["type"] != original[k]["type"]:
+            if (
+                k in override_all_if_type_changes
+                and "type" in value
+                and "type" in original[k]
+                and value["type"] != original[k]["type"]
+            ):
                 original[k] = value
             # Allowed key -> ok to add new subkeys.
             elif k in allow_new_subkey_list:
@@ -70,10 +74,12 @@ def deep_update(
     return original
 
 
-def flatten_dict(dt: Dict,
-                 delimiter: str = "/",
-                 prevent_delimiter: bool = False,
-                 flatten_list: bool = False):
+def flatten_dict(
+    dt: Dict,
+    delimiter: str = "/",
+    prevent_delimiter: bool = False,
+    flatten_list: bool = False,
+):
     """Flatten dict.
 
     Output and input are of the same dict type.
@@ -83,7 +89,8 @@ def flatten_dict(dt: Dict,
     def _raise_delimiter_exception():
         raise ValueError(
             f"Found delimiter `{delimiter}` in key when trying to flatten "
-            f"array. Please avoid using the delimiter in your specification.")
+            f"array. Please avoid using the delimiter in your specification."
+        )
 
     dt = copy.copy(dt)
     if prevent_delimiter and any(delimiter in key for key in dt):
@@ -133,13 +140,13 @@ def unflatten_dict(dt: Dict[str, T], delimiter: str = "/") -> Dict[str, T]:
                     f"Cannot unflatten dict due the key '{key}' "
                     f"having a parent key '{k}', which value is not "
                     f"of type {dict_type} (got {type(item)}). "
-                    "Change the key names to resolve the conflict.")
+                    "Change the key names to resolve the conflict."
+                )
         item[path[-1]] = val
     return out
 
 
-def unflatten_list_dict(dt: Dict[str, T],
-                        delimiter: str = "/") -> Dict[str, T]:
+def unflatten_list_dict(dt: Dict[str, T], delimiter: str = "/") -> Dict[str, T]:
     """Unflatten nested dict and list.
 
     This function now has some limitations:
@@ -163,8 +170,7 @@ def unflatten_list_dict(dt: Dict[str, T],
         >>> unflatten_list_dict(dt)
         {'aaa': [{'bb': 12}, {'cc': 56, 'dd': 92}]}
     """
-    out_type = list if list(dt)[0].split(delimiter, 1)[0].isdigit() \
-        else type(dt)
+    out_type = list if list(dt)[0].split(delimiter, 1)[0].isdigit() else type(dt)
     out = out_type()
     for key, val in dt.items():
         path = key.split(delimiter)
@@ -188,10 +194,9 @@ def unflatten_list_dict(dt: Dict[str, T],
     return out
 
 
-def unflattened_lookup(flat_key: str,
-                       lookup: Union[Mapping, Sequence],
-                       delimiter: str = "/",
-                       **kwargs) -> Union[Mapping, Sequence]:
+def unflattened_lookup(
+    flat_key: str, lookup: Union[Mapping, Sequence], delimiter: str = "/", **kwargs
+) -> Union[Mapping, Sequence]:
     """
     Unflatten `flat_key` and iteratively look up in `lookup`. E.g.
     `flat_key="a/0/b"` will try to return `lookup["a"][0]["b"]`.
