@@ -7,17 +7,22 @@ import uuid
 import json
 import weakref
 import ray
-from ray.util.inspect import (is_function_or_method, is_class_method,
-                              is_static_method)
+from ray.util.inspect import is_function_or_method, is_class_method, is_static_method
 from ray._private import signature
 
-from ray.workflow.common import (slugify, WorkflowData, Workflow, WorkflowRef,
-                                 StepType, WorkflowStepRuntimeOptions)
+from ray.workflow.common import (
+    slugify,
+    WorkflowData,
+    Workflow,
+    WorkflowRef,
+    StepType,
+    WorkflowStepRuntimeOptions,
+)
 from ray.workflow import serialization_context
 from ray.workflow.storage import Storage, get_global_storage
 from ray.workflow.workflow_storage import WorkflowStorage
 from ray.workflow.recovery import get_latest_output
-from ray.workflow.workflow_access import (get_or_create_management_actor)
+from ray.workflow.workflow_access import get_or_create_management_actor
 from ray.workflow import workflow_context
 from ray.workflow.step_executor import execute_workflow
 
@@ -64,16 +69,17 @@ class ActorMethod(ActorMethodBase):
         _method_name: The name of the actor method.
     """
 
-    def __init__(self, actor, method_name,
-                 method_helper: "_VirtualActorMethodHelper"):
+    def __init__(self, actor, method_name, method_helper: "_VirtualActorMethodHelper"):
         self._actor_ref = weakref.ref(actor)
         self._method_name = method_name
         self._method_helper = method_helper
 
     def __call__(self, *args, **kwargs):
-        raise TypeError("Actor methods cannot be called directly. Instead "
-                        f"of running 'object.{self._method_name}()', try "
-                        f"'object.{self._method_name}.remote()'.")
+        raise TypeError(
+            "Actor methods cannot be called directly. Instead "
+            f"of running 'object.{self._method_name}()', try "
+            f"'object.{self._method_name}.remote()'."
+        )
 
     def run_async(self, *args, **kwargs) -> "ObjectRef":
         """Execute the actor method asynchronously.
@@ -83,13 +89,15 @@ class ActorMethod(ActorMethodBase):
         """
         return self._run(args, kwargs)
 
-    def options(self,
-                *,
-                max_retries: int = 1,
-                catch_exceptions: bool = False,
-                name: str = None,
-                metadata: Dict[str, Any] = None,
-                **ray_options) -> ActorMethodBase:
+    def options(
+        self,
+        *,
+        max_retries: int = 1,
+        catch_exceptions: bool = False,
+        name: str = None,
+        metadata: Dict[str, Any] = None,
+        **ray_options,
+    ) -> ActorMethodBase:
         """This function set how the actor method is going to be executed.
 
         Args:
@@ -124,19 +132,23 @@ class ActorMethod(ActorMethodBase):
     def _run(self, args: Tuple[Any], kwargs: Dict[str, Any]) -> "ObjectRef":
         actor = self._actor_ref()
         if actor is None:
-            raise RuntimeError("Lost reference to actor. One common cause is "
-                               "doing 'workflow.get_actor(id).method.run()', "
-                               "in this case the actor instance is released "
-                               "by Python before calling the method. Try "
-                               "'actor = workflow.get_actor(id); "
-                               "actor.method.run()' instead.")
+            raise RuntimeError(
+                "Lost reference to actor. One common cause is "
+                "doing 'workflow.get_actor(id).method.run()', "
+                "in this case the actor instance is released "
+                "by Python before calling the method. Try "
+                "'actor = workflow.get_actor(id); "
+                "actor.method.run()' instead."
+            )
         try:
             return actor._actor_method_call(
-                self._method_helper, args=args, kwargs=kwargs)
+                self._method_helper, args=args, kwargs=kwargs
+            )
         except TypeError as exc:  # capture a friendlier stacktrace
             raise TypeError(
                 "Invalid input arguments for virtual actor "
-                f"method '{self._method_name}': {str(exc)}") from None
+                f"method '{self._method_name}': {str(exc)}"
+            ) from None
 
     def __getstate__(self):
         return {
@@ -146,8 +158,7 @@ class ActorMethod(ActorMethodBase):
         }
 
     def __setstate__(self, state):
-        self.__init__(state["actor"], state["method_name"],
-                      state["method_helper"])
+        self.__init__(state["actor"], state["method_name"], state["method_helper"])
 
 
 def __getstate(instance):
@@ -157,10 +168,12 @@ def __getstate(instance):
         try:
             state = json.dumps(instance.__dict__)
         except TypeError as e:
-            raise ValueError("The virtual actor contains fields that can't be "
-                             "converted to JSON. Please define `__getstate__` "
-                             "and `__setstate__` explicitly:"
-                             f" {instance.__dict__}") from e
+            raise ValueError(
+                "The virtual actor contains fields that can't be "
+                "converted to JSON. Please define `__getstate__` "
+                "and `__setstate__` explicitly:"
+                f" {instance.__dict__}"
+            ) from e
     return state
 
 
@@ -175,8 +188,13 @@ class _VirtualActorMethodHelper:
     """This is a helper class for managing options and creating workflow steps
     from raw class methods."""
 
-    def __init__(self, original_class, method: Callable, method_name: str,
-                 runtime_options: WorkflowStepRuntimeOptions):
+    def __init__(
+        self,
+        original_class,
+        method: Callable,
+        method_name: str,
+        runtime_options: WorkflowStepRuntimeOptions,
+    ):
         self._original_class = original_class
         self._original_method = method
         # Extract the signature of the method. This will be used
@@ -187,15 +205,15 @@ class _VirtualActorMethodHelper:
         # argument. For class and static methods, we do not want to bind
         # the first argument, but we do for instance methods
         method = inspect.unwrap(method)
-        is_bound = (is_class_method(method)
-                    or is_static_method(original_class, method_name))
+        is_bound = is_class_method(method) or is_static_method(
+            original_class, method_name
+        )
 
         # Print a warning message if the method signature is not
         # supported. We don't raise an exception because if the actor
         # inherits from a class that has a method whose signature we
         # don't support, there may not be much the user can do about it.
-        self._signature = signature.extract_signature(
-            method, ignore_first=not is_bound)
+        self._signature = signature.extract_signature(method, ignore_first=not is_bound)
 
         self._method = method
         self._method_name = method_name
@@ -223,15 +241,14 @@ class _VirtualActorMethodHelper:
                 state_ref = WorkflowRef(ws.get_entrypoint_step_id())
             # This is a hack to insert a positional argument.
             flattened_args = [signature.DUMMY_TYPE, state_ref] + flattened_args
-        workflow_inputs = serialization_context.make_workflow_inputs(
-            flattened_args)
+        workflow_inputs = serialization_context.make_workflow_inputs(flattened_args)
 
         if self.readonly:
             _actor_method = _wrap_readonly_actor_method(
-                actor_id, self._original_class, self._method_name)
+                actor_id, self._original_class, self._method_name
+            )
         else:
-            _actor_method = _wrap_actor_method(self._original_class,
-                                               self._method_name)
+            _actor_method = _wrap_actor_method(self._original_class, self._method_name)
         workflow_data = WorkflowData(
             func_body=_actor_method,
             inputs=workflow_inputs,
@@ -242,13 +259,15 @@ class _VirtualActorMethodHelper:
         wf = Workflow(workflow_data)
         return wf
 
-    def options(self,
-                *,
-                max_retries=1,
-                catch_exceptions=False,
-                name=None,
-                metadata=None,
-                **ray_options) -> "_VirtualActorMethodHelper":
+    def options(
+        self,
+        *,
+        max_retries=1,
+        catch_exceptions=False,
+        name=None,
+        metadata=None,
+        **ray_options,
+    ) -> "_VirtualActorMethodHelper":
         if metadata is not None:
             if not isinstance(metadata, dict):
                 raise ValueError("metadata must be a dict.")
@@ -258,17 +277,20 @@ class _VirtualActorMethodHelper:
                 except TypeError as e:
                     raise ValueError(
                         "metadata values must be JSON serializable, "
-                        "however '{}' has a value whose {}.".format(k, e))
+                        "however '{}' has a value whose {}.".format(k, e)
+                    )
         options = WorkflowStepRuntimeOptions.make(
             step_type=self._options.step_type,
             catch_exceptions=catch_exceptions,
             max_retries=max_retries,
-            ray_options=ray_options)
+            ray_options=ray_options,
+        )
         _self = _VirtualActorMethodHelper(
             self._original_class,
             self._original_method,
             self._method_name,
-            runtime_options=options)
+            runtime_options=options,
+        )
 
         _self._name = name
         _self._user_metadata = metadata
@@ -283,8 +305,7 @@ class VirtualActorMetadata:
     the signatures of its methods etc."""
 
     def __init__(self, original_class: type):
-        actor_methods = inspect.getmembers(original_class,
-                                           is_function_or_method)
+        actor_methods = inspect.getmembers(original_class, is_function_or_method)
 
         self.cls = original_class
         self.module = original_class.__module__
@@ -292,15 +313,15 @@ class VirtualActorMetadata:
         self.qualname = original_class.__qualname__
         self.methods = {}
         for method_name, method in actor_methods:
-            self._readonly = getattr(method, "__virtual_actor_readonly__",
-                                     False)
+            self._readonly = getattr(method, "__virtual_actor_readonly__", False)
             if self._readonly:
                 step_type = StepType.READONLY_ACTOR_METHOD
             else:
                 step_type = StepType.ACTOR_METHOD
             options = WorkflowStepRuntimeOptions.make(step_type=step_type)
             self.methods[method_name] = _VirtualActorMethodHelper(
-                original_class, method, method_name, runtime_options=options)
+                original_class, method, method_name, runtime_options=options
+            )
 
     def generate_random_actor_id(self) -> str:
         """Generate random actor ID."""
@@ -326,6 +347,7 @@ class VirtualActorClassBase(metaclass=abc.ABCMeta):
 
 class VirtualActorClass(VirtualActorClassBase):
     """The virtual actor class used to create a virtual actor."""
+
     _metadata: VirtualActorMetadata
 
     def __init__(self):
@@ -333,7 +355,8 @@ class VirtualActorClass(VirtualActorClassBase):
         # an actor class if this was meant to be subclassed.
         assert False, (
             "VirtualActorClass.__init__ should not be called. Please use "
-            "the @workflow.actor decorator instead.")
+            "the @workflow.actor decorator instead."
+        )
 
     def __call__(self, *args, **kwargs):
         """Prevents users from directly instantiating an ActorClass.
@@ -346,9 +369,11 @@ class VirtualActorClass(VirtualActorClassBase):
         Raises:
             Exception: Always.
         """
-        raise TypeError("Actors cannot be instantiated directly. "
-                        f"Instead of '{self._metadata.name}()', "
-                        f"use '{self._metadata.name}.create()'.")
+        raise TypeError(
+            "Actors cannot be instantiated directly. "
+            f"Instead of '{self._metadata.name}()', "
+            f"use '{self._metadata.name}.create()'."
+        )
 
     @classmethod
     def _from_class(cls, base_class: type) -> "VirtualActorClass":
@@ -356,16 +381,18 @@ class VirtualActorClass(VirtualActorClassBase):
         # TODO(suquark): we may use more complex name for private functions
         # to avoid collision with user-defined functions.
         for attribute in [
-                "create",
-                "_create",
-                "option",
-                "_construct",
-                "_from_class",
+            "create",
+            "_create",
+            "option",
+            "_construct",
+            "_from_class",
         ]:
             if hasattr(base_class, attribute):
-                logger.warning("Creating an actor from class "
-                               f"{base_class.__name__} overwrites "
-                               f"attribute {attribute} of that class")
+                logger.warning(
+                    "Creating an actor from class "
+                    f"{base_class.__name__} overwrites "
+                    f"attribute {attribute} of that class"
+                )
 
         if not is_function_or_method(getattr(base_class, "__init__", None)):
             # Add __init__ if it does not exist.
@@ -408,7 +435,8 @@ class VirtualActorClass(VirtualActorClassBase):
     def get_or_create(self, actor_id: str, *args, **kwargs) -> "VirtualActor":
         """Create an actor. See `VirtualActorClassBase.create()`."""
         return self._get_or_create(
-            actor_id, args=args, kwargs=kwargs, storage=get_global_storage())
+            actor_id, args=args, kwargs=kwargs, storage=get_global_storage()
+        )
 
     # TODO(suquark): support num_cpu etc in options
     def options(self) -> VirtualActorClassBase:
@@ -419,15 +447,14 @@ class VirtualActorClass(VirtualActorClassBase):
         class ActorOptionWrapper(VirtualActorClassBase):
             def get_or_create(self, actor_id: str, *args, **kwargs):
                 return actor_cls._get_or_create(
-                    actor_id,
-                    args=args,
-                    kwargs=kwargs,
-                    storage=get_global_storage())
+                    actor_id, args=args, kwargs=kwargs, storage=get_global_storage()
+                )
 
         return ActorOptionWrapper()
 
-    def _get_or_create(self, actor_id: str, args, kwargs,
-                       storage: Storage) -> "VirtualActor":
+    def _get_or_create(
+        self, actor_id: str, args, kwargs, storage: Storage
+    ) -> "VirtualActor":
         """Create a new virtual actor"""
         try:
             return get_actor(actor_id, storage)
@@ -441,7 +468,7 @@ class VirtualActorClass(VirtualActorClassBase):
         return VirtualActor(self._metadata, actor_id, storage)
 
     def __reduce__(self):
-        return decorate_actor, (self._metadata.cls, )
+        return decorate_actor, (self._metadata.cls,)
 
 
 def _wrap_readonly_actor_method(actor_id: str, cls: type, method_name: str):
@@ -456,7 +483,8 @@ def _wrap_readonly_actor_method(actor_id: str, cls: type, method_name: str):
             raise VirtualActorNotInitializedError(
                 f"Virtual actor '{actor_id}' has not been initialized. "
                 "We cannot get the latest state for the "
-                "readonly virtual actor.") from e
+                "readonly virtual actor."
+            ) from e
         __setstate(instance, state)
         method = getattr(instance, method_name)
         return method(*args, **kwargs)
@@ -490,8 +518,7 @@ def _wrap_actor_method(cls: type, method_name: str):
 class VirtualActor:
     """The instance of a virtual actor class."""
 
-    def __init__(self, metadata: VirtualActorMetadata, actor_id: str,
-                 storage: Storage):
+    def __init__(self, metadata: VirtualActorMetadata, actor_id: str, storage: Storage):
         self._metadata = metadata
         self._actor_id = actor_id
         self._storage = storage
@@ -519,14 +546,15 @@ class VirtualActor:
 
     def __getattr__(self, method_name):
         if method_name in self._metadata.methods:
-            return ActorMethod(self, method_name,
-                               self._metadata.methods[method_name])
+            return ActorMethod(self, method_name, self._metadata.methods[method_name])
         raise AttributeError(f"No method with name '{method_name}'")
 
-    def _actor_method_call(self, method_helper: _VirtualActorMethodHelper,
-                           args, kwargs) -> "ObjectRef":
-        with workflow_context.workflow_step_context(self._actor_id,
-                                                    self._storage.storage_url):
+    def _actor_method_call(
+        self, method_helper: _VirtualActorMethodHelper, args, kwargs
+    ) -> "ObjectRef":
+        with workflow_context.workflow_step_context(
+            self._actor_id, self._storage.storage_url
+        ):
             wf = method_helper.step(*args, **kwargs)
             if method_helper.readonly:
                 return execute_workflow(wf).volatile_output
